#ifndef CUETRACKWIDGET_H
#define CUETRACKWIDGET_H

#include <QWidget>
#include <QLineEdit>
#include <QSpinBox>
#include <QComboBox>
#include <QFormLayout>
#include <QCheckBox>
#include <QLabel>
#include <QPushButton>
#include <QDialog>
#include "defines.h"
#include "clickabledoublespinbox.h"

class CueTrackWidget : public QWidget {
    Q_OBJECT

public:
    explicit CueTrackWidget(QWidget *parent = nullptr);
    CueTrack saveCueTrack() const;
    void loadCueTrack(const CueTrack &cueTrack);
    int getVolume() const { return volumeSpin->value() * 2.55f * 255.0f; }
    int getPan() const { return (panSpin->value() * 128) + 128; }
    int getPitch() const { return pitchSpin->value() * 128; }
    int getBus1() const { return bus1Spin->value() * 2.55f * 255.0f; }
    int getBus2() const { return bus2Spin->value() * 2.55f * 255.0f; }
	Status getStatus() const { return static_cast<Status>(statusCombo->currentIndex()); }
	QString getFilePath() const { return filePathEdit->text(); }
    void setFilePath(QString text) const { filePathEdit->setText(text);};
	int getFadeIn() const { return fadeInSpin->value(); }
	int getFadeOut() const { return fadeOutSpin->value(); }
	int getWaitIn() const { return waitInSpin->value(); }
	int getWaitOut() const { return waitOutSpin->value(); }
	bool getStopAtEnd() const { return stopAtEndCheck->isChecked(); }
	QString getName() const { return nameEdit->text(); }
	QString getDescription() const { return descriptionEdit->text(); }
	int getUserNumber() const { return userNumberSpin->value(); }
	int getEntryPoint() const { return entryPointSpin->value(); }
	int getExitPoint() const { return exitPointSpin->value(); }
    void setStyle(bool highlight);

    QCheckBox* active;
    QSpinBox* audioLayer;
    QLineEdit* filePathEdit;
    ClickableDoubleSpinBox* volumeSpin;
    ClickableDoubleSpinBox* panSpin;
    ClickableDoubleSpinBox* pitchSpin;
    ClickableDoubleSpinBox* bus1Spin;
    ClickableDoubleSpinBox* bus2Spin;
    QComboBox* statusCombo;
    QSpinBox* fadeOutSpin;
    QSpinBox* fadeInSpin;
    QSpinBox* waitInSpin;
    QSpinBox* waitOutSpin;
    QCheckBox* stopAtEndCheck;
    QLineEdit* nameEdit;
    QLineEdit* descriptionEdit;
    QSpinBox* userNumberSpin;
    QSpinBox* entryPointSpin;
    QSpinBox* exitPointSpin;
	QPushButton* editButton;
    

private:
    void setupUi();
    void setupStatusCombo();

private slots:
    void editButtonClicked();
};

// Definición de la clase EditCueTrackWidget
#include <QFileDialog>
#include "settings.h"

class EditCueTrackWidget : public QDialog {
	Q_OBJECT

public:
	explicit EditCueTrackWidget(QWidget *parent = nullptr);

	void loadCueTrack(const CueTrack &cueTrack);
	CueTrack saveCueTrack() const;

private:
	void setupUi();
	void connectSignals();
    void onBrowseButtonClicked();

    QLineEdit *filePathEdit;
	QDoubleSpinBox *volumeSpin;
	QDoubleSpinBox *panSpin;
	QDoubleSpinBox *pitchSpin;
	QDoubleSpinBox *bus1Spin;
	QDoubleSpinBox *bus2Spin;
	QSpinBox *fadeInSpin;
	QSpinBox *fadeOutSpin;
	QSpinBox *waitInSpin;
	QSpinBox *waitOutSpin;
	QCheckBox *stopAtEndCheck;
	QLineEdit *nameEdit;
	QLineEdit *descriptionEdit;
	QSpinBox *userNumberSpin;
	QSpinBox *entryPointSpin;
	QSpinBox *exitPointSpin;
	QComboBox *statusCombo;
	QPushButton *browseButton;
	QPushButton *saveButton;
	QPushButton *cancelButton;

	CueTrack currentCueTrack;
};

EditCueTrackWidget::EditCueTrackWidget(QWidget *parent) : QDialog(parent) {
	setupUi();
	connectSignals();
}

void EditCueTrackWidget::setupUi() {
	filePathEdit = new QLineEdit(this);
	volumeSpin = new QDoubleSpinBox(this);
	panSpin = new QDoubleSpinBox(this);
	pitchSpin = new QDoubleSpinBox(this);
	bus1Spin = new QDoubleSpinBox(this);
	bus2Spin = new QDoubleSpinBox(this);
	fadeInSpin = new QSpinBox(this);
	fadeOutSpin = new QSpinBox(this);
	waitInSpin = new QSpinBox(this);
	waitOutSpin = new QSpinBox(this);
	stopAtEndCheck = new QCheckBox(this);
	nameEdit = new QLineEdit(this);
	descriptionEdit = new QLineEdit(this);
	userNumberSpin = new QSpinBox(this);
	entryPointSpin = new QSpinBox(this);
	exitPointSpin = new QSpinBox(this);
	statusCombo = new QComboBox(this);
	statusCombo->addItem("Stopped");
	statusCombo->addItem("Paused");
	statusCombo->addItem("PlayingOnce");
	statusCombo->addItem("PlayingLoop");
	statusCombo->addItem("Iddle");
	statusCombo->addItem("PlayingFolder");
	statusCombo->addItem("PlayingFolderLoop");
	statusCombo->addItem("PlayingFolderRandom");

	browseButton = new QPushButton("Browse...", this);
	saveButton = new QPushButton("Save", this);
	cancelButton = new QPushButton("Cancel", this);

	QFormLayout *layout = new QFormLayout(this);
	layout->addRow("User Number", userNumberSpin);
	layout->addRow("Name", nameEdit);
	layout->addRow("File Path", filePathEdit);
	layout->addRow(browseButton);
	layout->addRow("Status", statusCombo);
	layout->addRow("Fade In", fadeInSpin);
	layout->addRow("Fade Out", fadeOutSpin);
	layout->addRow("Wait In", waitInSpin);
	layout->addRow("Wait Out", waitOutSpin);
	layout->addRow("Stop At End", stopAtEndCheck);
	layout->addRow("Volume", volumeSpin);
	layout->addRow("Bus 1", bus1Spin);
	layout->addRow("Bus 2", bus2Spin);
	layout->addRow("Pan", panSpin);
	layout->addRow("Pitch", pitchSpin);
	layout->addRow("Entry Point", entryPointSpin);
	layout->addRow("Exit Point", exitPointSpin);
	layout->addRow("Description", descriptionEdit);
	layout->addRow(saveButton);
	layout->addRow(cancelButton);
}

void EditCueTrackWidget::loadCueTrack(const CueTrack &cueTrack) {
    filePathEdit->setText(cueTrack.filePath.data());
	volumeSpin->setValue(cueTrack.volume);
	panSpin->setValue(cueTrack.pan);
	pitchSpin->setValue(cueTrack.pitch);
	bus1Spin->setValue(cueTrack.bus1);
	bus2Spin->setValue(cueTrack.bus2);
	fadeInSpin->setValue(cueTrack.fadeIn);
	fadeOutSpin->setValue(cueTrack.fadeOut);
	waitInSpin->setValue(cueTrack.waitIn);
	waitOutSpin->setValue(cueTrack.waitOut);
	stopAtEndCheck->setChecked(cueTrack.stopAtEnd);
    nameEdit->setText(cueTrack.name.data());
    descriptionEdit->setText(cueTrack.description.data());
	userNumberSpin->setValue(cueTrack.userNumber);
	entryPointSpin->setValue(cueTrack.entryPoint);
	exitPointSpin->setValue(cueTrack.exitPoint);
	statusCombo->setCurrentIndex(statusCombo->findText(statusToString(cueTrack.status)));	
}

CueTrack EditCueTrackWidget::saveCueTrack() const {
	CueTrack cueTrack;
    cueTrack.filePath.append(filePathEdit->text().toUtf8().constData());
	cueTrack.volume = volumeSpin->value();
	cueTrack.pan = panSpin->value();
	cueTrack.pitch = pitchSpin->value();
    cueTrack.name.append(nameEdit->text().toUtf8().constData());
    cueTrack.description.append(descriptionEdit->text().toUtf8().constData());
	cueTrack.status = static_cast<Status>(statusCombo->currentIndex());
	cueTrack.fadeIn = fadeInSpin->value();
	cueTrack.fadeOut = fadeOutSpin->value();
	cueTrack.waitIn = waitInSpin->value();
	cueTrack.waitOut = waitOutSpin->value();
	cueTrack.stopAtEnd = stopAtEndCheck->isChecked();
	cueTrack.userNumber = userNumberSpin->value();
	cueTrack.entryPoint = entryPointSpin->value();
	cueTrack.exitPoint = exitPointSpin->value();
	return cueTrack;
}

void EditCueTrackWidget::connectSignals() {
	connect(saveButton, &QPushButton::clicked, this, &EditCueTrackWidget::accept);
	connect(cancelButton, &QPushButton::clicked, this, &EditCueTrackWidget::reject);
    connect(browseButton, &QPushButton::clicked, this, &EditCueTrackWidget::onBrowseButtonClicked);
}

void EditCueTrackWidget::onBrowseButtonClicked() {
    QString filePath = QFileDialog::getOpenFileName(this, tr("Select File"), Settings::getInstance()->getPathMedia(), tr("Audio Files (*.mp3 *.wav *.flac)"));
    if (!filePath.isEmpty()) {
        filePathEdit->setText(filePath);
        QDir dir;
        Settings::getInstance()->setPathMedia(dir.absoluteFilePath(filePath));
    }
}

#endif // CUETRACKWIDGET_H
